package es.uv.garcosda.apiusers.controllers;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;

import es.uv.garcosda.shared.User;

@RestController
@RequestMapping("/api/v1/users")
public class UsersController {

	
	private RestTemplate template;
	
	@Value("${api.uri.users}")
	private String usersApi;
	
	public UsersController() {
		this.template = new RestTemplate();
	}
	
	@GetMapping()
	public ResponseEntity<?> findUsers(@RequestParam(value="firstname", required = false) Optional<String> firstname, 
			@RequestParam(value="lastname", required = false) Optional<String> lastname) {
		
		String url = usersApi;
		if(firstname.isPresent()) {
			url+="?firstname="+firstname.get();
		}
		else if(lastname.isPresent()) {
			url+="?lastname="+lastname.get();
		}	
		
		try {
			ResponseEntity<User[]> response = template.getForEntity(url, User[].class); 
			if(response.getStatusCode() == HttpStatus.OK) {
				List<User> Users = Arrays.stream(response.getBody()).collect(Collectors.toList());
				return new ResponseEntity<List<User>>(Users, HttpStatus.OK);
			}
			return new ResponseEntity<String>("API returned no content", HttpStatus.SERVICE_UNAVAILABLE);
		}
		catch(ResourceAccessException e){ }
		
		return new ResponseEntity<String>("Cannot connect to repository", HttpStatus.SERVICE_UNAVAILABLE);		
	}
	
	@GetMapping("{id}")
	public ResponseEntity<?> findUserById(@PathVariable("id") String id){
		try {
			User v = template.getForObject(usersApi+"/"+id, User.class);
			if(v.getId() != null) {
				return new ResponseEntity<User>(v, HttpStatus.OK);	
			}
			return new ResponseEntity<String>("User not found", HttpStatus.NOT_FOUND);	
		}
		catch(ResourceAccessException e){ }
		
		return new ResponseEntity<String>("Cannot connect to repository", HttpStatus.SERVICE_UNAVAILABLE);	
	}
	
	@PostMapping()
	public ResponseEntity<?> createUser(@RequestBody User User) {
		try {
			ResponseEntity<User> response = template.postForEntity(usersApi, User, User.class);
			return response;
		}
		catch(ResourceAccessException e){ }
		
		return new ResponseEntity<String>("Cannot connect to repository", HttpStatus.SERVICE_UNAVAILABLE);
	}
	
	@DeleteMapping("{id}")
	public ResponseEntity<String> deleteUserById(@PathVariable("id") String id) {
		try {
			template.delete(usersApi+"/"+id);
			return new ResponseEntity<String>("User deleted", HttpStatus.OK);
		}
		catch(ResourceAccessException e){ }
		
		return new ResponseEntity<String>("Cannot connect to repository", HttpStatus.SERVICE_UNAVAILABLE);
	}
	
	
	
}
